@@ -53,6 +53,7 @@
 #include "WebKitWebContextPrivate.h"
 #include "WebKitWebViewPrivate.h"
 #include "WebKitWebsiteDataManagerPrivate.h"
+#include "WebKitWebsitePoliciesPrivate.h"
 #include "WebNotificationManagerProxy.h"
 #include "WebProcessMessages.h"
 #include "WebURLSchemeHandler.h"
@@ -69,6 +70,7 @@
 #include <wtf/NeverDestroyed.h>
 #include <wtf/RefCounted.h>
 #include <wtf/RefPtr.h>
+#include <wtf/URLParser.h>
 #include <wtf/glib/GRefPtr.h>
 #include <wtf/glib/GUniquePtr.h>
 #include <wtf/glib/WTFGType.h>
@@ -120,7 +122,10 @@ enum {
 #endif
     PROP_WEBSITE_DATA_MANAGER,
 #if PLATFORM(GTK)
-    PROP_PSON_ENABLED
+    PROP_PSON_ENABLED,
+#if !USE(GTK4)
+    PROP_USE_SYSYEM_APPEARANCE_FOR_SCROLLBARS
+#endif
 #endif
 };
 
@@ -204,6 +209,9 @@ struct _WebKitWebContextPrivate {
     bool clientsDetached;
 #if PLATFORM(GTK)
     bool psonEnabled;
+#if !USE(GTK4)
+    bool useSystemAppearanceForScrollbars;
+#endif
 #endif
 
     GRefPtr<WebKitFaviconDatabase> faviconDatabase;
@@ -333,6 +341,11 @@ static void webkitWebContextGetProperty(GObject* object, guint propID, GValue* v
     case PROP_PSON_ENABLED:
         g_value_set_boolean(value, context->priv->psonEnabled);
         break;
+#if !USE(GTK4)
+    case PROP_USE_SYSYEM_APPEARANCE_FOR_SCROLLBARS:
+        g_value_set_boolean(value, webkit_web_context_get_use_system_appearance_for_scrollbars(context));
+        break;
+#endif
 #endif
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, propID, paramSpec);
@@ -358,6 +371,11 @@ static void webkitWebContextSetProperty(GObject* object, guint propID, const GVa
     case PROP_PSON_ENABLED:
         context->priv->psonEnabled = g_value_get_boolean(value);
         break;
+#if !USE(GTK4)
+    case PROP_USE_SYSYEM_APPEARANCE_FOR_SCROLLBARS:
+        webkit_web_context_set_use_system_appearance_for_scrollbars(context, g_value_get_boolean(value));
+        break;
+#endif
 #endif
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, propID, paramSpec);
@@ -377,18 +395,16 @@ static void webkitWebContextConstructed(GObject* object)
     configuration.setInjectedBundlePath(FileSystem::stringFromFileSystemRepresentation(bundleFilename.get()));
 #if PLATFORM(GTK)
     configuration.setProcessSwapsOnNavigation(priv->psonEnabled);
-    if (!priv->psonEnabled) {
-        const char* useSingleWebProcess = getenv("WEBKIT_USE_SINGLE_WEB_PROCESS");
-        if (useSingleWebProcess && strcmp(useSingleWebProcess, "0"))
-            configuration.setUsesSingleWebProcess(true);
-    }
+#if !USE(GTK4)
+    configuration.setUseSystemAppearanceForScrollbars(priv->useSystemAppearanceForScrollbars);
+#endif
 #endif
 
     if (!priv->websiteDataManager)
         priv->websiteDataManager = adoptGRef(webkit_website_data_manager_new("local-storage-directory", priv->localStorageDirectory.data(), nullptr));
 
     if (!webkit_website_data_manager_is_ephemeral(priv->websiteDataManager.get()))
-        WebKit::LegacyGlobalSettings::singleton().setHSTSStorageDirectory(FileSystem::stringFromFileSystemRepresentation(webkit_website_data_manager_get_hsts_cache_directory(priv->websiteDataManager.get())));
+        configuration.setHSTSStorageDirectory(FileSystem::stringFromFileSystemRepresentation(webkit_website_data_manager_get_hsts_cache_directory(priv->websiteDataManager.get())));
 
     priv->processPool = WebProcessPool::create(configuration);
     priv->processPool->setPrimaryDataStore(webkitWebsiteDataManagerGetDataStore(priv->websiteDataManager.get()));
@@ -524,6 +540,29 @@ static void webkit_web_context_class_init(WebKitWebContextClass* webContextClass
             _("Whether swap Web processes on cross-site navigations is enabled"),
             FALSE,
             static_cast<GParamFlags>(WEBKIT_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
+
+#if !USE(GTK4)
+    /**
+     * WebKitWebContext:use-system-appearance-for-scrollbars:
+     *
+     * Whether to use system appearance for rendering scrollbars.
+     *
+     * This is enabled by default for backwards compatibility, but it's only
+     * recommened to use when the application includes other widgets to ensure
+     * consistency, or when consistency with other applications is required too.
+     *
+     * Since: 2.30
+     */
+    g_object_class_install_property(
+        gObjectClass,
+        PROP_USE_SYSYEM_APPEARANCE_FOR_SCROLLBARS,
+        g_param_spec_boolean(
+            "use-system-appearance-for-scrollbars",
+            _("Use system appearance for scrollbars"),
+            _("Whether to use system appearance for rendering scrollbars"),
+            TRUE,
+            static_cast<GParamFlags>(WEBKIT_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+#endif
 #endif
 
     /**
@@ -1236,13 +1275,16 @@ void webkit_web_context_register_uri_scheme(WebKitWebContext* context, const cha
     g_return_if_fail(scheme);
     g_return_if_fail(callback);
 
-    // List from Source/WTF/URLParser.cpp, enum Scheme.
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "ws") != 0);
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "wss") != 0);
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "file") != 0);
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "ftp") != 0);
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "http") != 0);
-    g_return_if_fail(g_ascii_strcasecmp(scheme, "https") != 0);
+    auto canonicalizedScheme = WTF::URLParser::maybeCanonicalizeScheme(scheme);
+    if (!canonicalizedScheme) {
+        g_critical("Cannot register invalid URI scheme %s", scheme);
+        return;
+    }
+
+    if (WTF::URLParser::isSpecialScheme(canonicalizedScheme.value())) {
+        g_warning("Registering special URI scheme %s is no longer allowed", scheme);
+        return;
+    }
 
     auto handler = WebKitURISchemeHandler::create(context, callback, userData, destroyNotify);
     auto addResult = context->priv->uriSchemeHandlers.set(String::fromUTF8(scheme), WTFMove(handler));
@@ -1274,9 +1316,9 @@ void webkit_web_context_set_sandbox_enabled(WebKitWebContext* context, gboolean
     context->priv->processPool->setSandboxEnabled(enabled);
 }
 
-static bool pathIsBlacklisted(const char* path)
+static bool pathIsBlocked(const char* path)
 {
-    static const Vector<CString, 4> blacklistedPrefixes = {
+    static const Vector<CString, 4> blockedPrefixes = {
         // These are recreated by bwrap and it doesn't make sense to try and rebind them.
         "sys", "proc", "dev",
         "", // All of `/` isn't acceptable.
@@ -1286,7 +1328,7 @@ static bool pathIsBlacklisted(const char* path)
         return true;
 
     GUniquePtr<char*> splitPath(g_strsplit(path, G_DIR_SEPARATOR_S, 3));
-    return blacklistedPrefixes.contains(splitPath.get()[1]);
+    return blockedPrefixes.contains(splitPath.get()[1]);
 }
 
 /**
@@ -1310,7 +1352,7 @@ void webkit_web_context_add_path_to_sandbox(WebKitWebContext* context, const cha
 {
     g_return_if_fail(WEBKIT_IS_WEB_CONTEXT(context));
 
-    if (pathIsBlacklisted(path)) {
+    if (pathIsBlocked(path)) {
         g_critical("Attempted to add disallowed path to sandbox: %s", path);
         return;
     }
@@ -1768,6 +1810,51 @@ void webkit_web_context_send_message_to_all_extensions(WebKitWebContext* context
         process->send(Messages::WebProcess::SendMessageToWebExtension(webkitUserMessageGetMessage(message)), 0);
 }
 
+#if PLATFORM(GTK) && !USE(GTK4)
+/**
+ * webkit_web_context_set_use_system_appearance_for_scrollbars:
+ * @context: a #WebKitWebContext
+ * @enabled: value to set
+ *
+ * Set the #WebKitWebContext:use-system-appearance-for-scrollbars property.
+ *
+ * Since: 2.30
+ */
+void webkit_web_context_set_use_system_appearance_for_scrollbars(WebKitWebContext* context, gboolean enabled)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_CONTEXT(context));
+
+    if (context->priv->useSystemAppearanceForScrollbars == enabled)
+        return;
+
+    context->priv->useSystemAppearanceForScrollbars = enabled;
+    g_object_notify(G_OBJECT(context), "use-system-appearance-for-scrollbars");
+
+    if (!context->priv->processPool)
+        return;
+
+    context->priv->processPool->configuration().setUseSystemAppearanceForScrollbars(enabled);
+    context->priv->processPool->sendToAllProcesses(Messages::WebProcess::SetUseSystemAppearanceForScrollbars(enabled));
+}
+
+/**
+ * webkit_web_context_get_use_system_appearance_for_scrollbars:
+ * @context: a #WebKitWebContext
+ *
+ * Get the #WebKitWebContext:use-system-appearance-for-scrollbars property.
+ *
+ * Returns: %TRUE if scrollbars are rendering using the system appearance, or %FALSE otherwise
+ *
+ * Since: 2.30
+ */
+gboolean webkit_web_context_get_use_system_appearance_for_scrollbars(WebKitWebContext* context)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_CONTEXT(context), TRUE);
+
+    return context->priv->useSystemAppearanceForScrollbars;
+}
+#endif
+
 void webkitWebContextInitializeNotificationPermissions(WebKitWebContext* context)
 {
     g_signal_emit(context, signals[INITIALIZE_NOTIFICATION_PERMISSIONS], 0);
@@ -1787,7 +1874,8 @@ WebKitDownload* webkitWebContextGetOrCreateDownload(DownloadProxy* downloadProxy
 WebKitDownload* webkitWebContextStartDownload(WebKitWebContext* context, const char* uri, WebPageProxy* initiatingPage)
 {
     WebCore::ResourceRequest request(String::fromUTF8(uri));
-    return webkitWebContextGetOrCreateDownload(&context->priv->processPool->download(WebKit::WebsiteDataStore::defaultDataStore().get(), initiatingPage, request));
+    auto& websiteDataStore = webkitWebsiteDataManagerGetDataStore(context->priv->websiteDataManager.get());
+    return webkitWebContextGetOrCreateDownload(&context->priv->processPool->download(websiteDataStore, initiatingPage, request));
 }
 
 void webkitWebContextRemoveDownload(DownloadProxy* downloadProxy)
@@ -1815,7 +1903,7 @@ WebProcessPool& webkitWebContextGetProcessPool(WebKitWebContext* context)
     return *context->priv->processPool;
 }
 
-void webkitWebContextCreatePageForWebView(WebKitWebContext* context, WebKitWebView* webView, WebKitUserContentManager* userContentManager, WebKitWebView* relatedView)
+void webkitWebContextCreatePageForWebView(WebKitWebContext* context, WebKitWebView* webView, WebKitUserContentManager* userContentManager, WebKitWebView* relatedView, WebKitWebsitePolicies* defaultWebsitePolicies)
 {
     auto pageConfiguration = API::PageConfiguration::create();
     pageConfiguration->setProcessPool(context->priv->processPool.get());
@@ -1828,6 +1916,7 @@ void webkitWebContextCreatePageForWebView(WebKitWebContext* context, WebKitWebVi
     if (!manager)
         manager = context->priv->websiteDataManager.get();
     pageConfiguration->setWebsiteDataStore(&webkitWebsiteDataManagerGetDataStore(manager));
+    pageConfiguration->setDefaultWebsitePolicies(webkitWebsitePoliciesGetWebsitePolicies(defaultWebsitePolicies));
     webkitWebViewCreatePage(webView, WTFMove(pageConfiguration));
 
     auto& page = webkitWebViewGetPage(webView);
