@@ -21,6 +21,7 @@
 #include "WebKitColor.h"
 
 #include "WebKitColorPrivate.h"
+#include <WebCore/CSSParser.h>
 
 /**
  * SECTION: WebKitColor
@@ -74,16 +75,14 @@ G_DEFINE_BOXED_TYPE(WebKitColor, webkit_color, webkit_color_copy, webkit_color_f
 
 const WebCore::Color webkitColorToWebCoreColor(WebKitColor* color)
 {
-    return WebCore::Color(static_cast<float>(color->red), static_cast<float>(color->green),
-        static_cast<float>(color->blue), static_cast<float>(color->alpha));
+    return WebCore::convertToComponentBytes(WebCore::SRGBA { static_cast<float>(color->red), static_cast<float>(color->green), static_cast<float>(color->blue), static_cast<float>(color->alpha) });
 }
 
 void webkitColorFillFromWebCoreColor(const WebCore::Color& webCoreColor, WebKitColor* color)
 {
     RELEASE_ASSERT(webCoreColor.isValid());
 
-    double r, g, b, a;
-    webCoreColor.getRGBA(r, g, b, a);
+    auto [r, g, b, a] = webCoreColor.toSRGBALossy<float>();
     color->red = r;
     color->green = g;
     color->blue = b;
@@ -109,7 +108,7 @@ gboolean webkit_color_parse(WebKitColor* color, const gchar* colorString)
     g_return_val_if_fail(color, FALSE);
     g_return_val_if_fail(colorString, FALSE);
 
-    auto webCoreColor = WebCore::Color(colorString);
+    auto webCoreColor = WebCore::CSSParser::parseColor({ colorString });
     if (!webCoreColor.isValid())
         return FALSE;
 
