@@ -28,6 +28,7 @@
 #include <QQuickWindow>
 #include <QSGSimpleTextureNode>
 #include <QScreen>
+#include <QtGlobal>
 #include <QtPlatformHeaders/QEGLNativeContext>
 #include <qpa/qplatformnativeinterface.h>
 #include <wtf/glib/GUniquePtr.h>
@@ -184,7 +185,12 @@ QSGNode* WPEQtView::updatePaintNode(QSGNode* node, UpdatePaintNodeData*)
     if (!textureId)
         return node;
 
-    textureNode->setTexture(window()->createTextureFromId(textureId, m_size.toSize(), QQuickWindow::TextureHasAlphaChannel));
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+    auto texture = window()->createTextureFromNativeObject(QQuickWindow::NativeObjectTexture, &textureId, 0, m_size.toSize(), QQuickWindow::TextureHasAlphaChannel);
+#else
+    auto texture = window()->createTextureFromId(textureId, m_size.toSize(), QQuickWindow::TextureHasAlphaChannel);
+#endif
+    textureNode->setTexture(texture);
     textureNode->setRect(boundingRect());
     return textureNode;
 }
